import { async as subject } from 'most-subject';
import { nextPayloadID, nextEventID, toTag, counter } from './id';
import { EventConstructor } from './event';
import { safeDispatch, port } from './port';
import { basicCommon, observable } from './event-common';
export function EffectConstructor(domainName, dispatch, getState, state$, events, name, opts, action$ = subject()) {
  var handlers = new Set();
  var {
    eventID,
    nextSeq,
    getType
  } = basicCommon(domainName, name);
  handlers.add((payload, state) => action$.next(payload)); // handlers.add((payload, state) => state$.next(state))

  var using = getWarn({
    domainName,
    name,
    mode: () => opts.effectImplementationCheck()
  });

  function use(thunk) {
    using = thunk;
  }

  var create = payload => ({
    type: getType(),
    payload,
    meta: {
      index: nextPayloadID(),
      eventID,
      seq: nextSeq(),
      passed: false
    }
  });

  var effect = params => {
    var run = () => {
      run = () => {};

      using(params).then(result => {
        var data = {
          params,
          result
        };
        dispatcher(done(data), dispatch);
        resolve(data);
      }, error => {
        var data = {
          params,
          error
        };
        dispatcher(fail(data), dispatch);
        reject(data);
      });
    };

    var runner = dispatchHook => {
      if (canDispatch) {
        canDispatch = false;
        dispatcher(result, dispatch, dispatchHook);

        for (var handler of handlers) {
          handler(params, getState());
        } // action$.next(params)


        run();
      }
    };

    var resolve = val => {};

    var reject = val => {};

    var doneP = new Promise(rs => resolve = rs);
    var failP = new Promise(rs => reject = rs);
    var toSend = Promise.resolve(params);
    var result = create(params);
    var canDispatch = true;
    return Object.assign({}, result, {
      raw() {
        return result;
      },

      send(dispatchHook) {
        runner(dispatchHook);
        return toSend;
      },

      done() {
        runner();
        return doneP;
      },

      fail() {
        runner();
        return failP;
      },

      promise() {
        runner();
        return Promise.race([doneP, failP.then(data => Promise.reject(data))]);
      }

    });
  };

  var done = EventConstructor(domainName, dispatch, getState, state$, events, [name, 'done'].join(' '));
  var fail = EventConstructor(domainName, dispatch, getState, state$, events, [name, 'fail'].join(' '));

  function watchFail(error) {
    watchFailWarn({
      name,
      domainName,
      mode: opts.watchFailCheck()
    }, error);
  }

  function justDispatch(result) {
    return dispatcher(result, dispatch);
  }

  function watchHandler(fn, params, state) {
    var result;

    try {
      result = fn(params, state);
    } catch (error) {
      watchFail(error);
      return;
    }

    if (params === result) {
      return;
    }

    if (result === 'object' && result != null && typeof result.then === 'function') {
      result.then(result => {
        if (result !== params) {
          justDispatch(result);
        }
      }, watchFail);
      return;
    }

    justDispatch(result);
  }

  function watch(fn) {
    handlers.add((params, state) => {
      watchHandler(fn, params, state);
    });
  }

  effect.use = use;
  effect.getType = getType;
  setToString(effect, getType);
  effect.done = done;
  effect.fail = fail;
  effect.watch = watch;
  effect.epic = port(dispatch, state$, action$);

  effect.trigger = function trigger(query, eventName = 'trigger') {
    var triggerEvent = EventConstructor(domainName, dispatch, getState, state$, events, [name, eventName].join(' '));
    triggerEvent.watch((_, state) => {
      var queryResult = query(state);
      return effect(queryResult);
    });
    return triggerEvent;
  }; // effect.port = function port<R>(events$: Stream<R>) {
  //   return events$.observe(data => safeDispatch(data, dispatch))
  // }


  effect.subscribe = subscriber => action$.subscribe(subscriber);

  observable(effect, action$);
  return effect;
}

function setToString(effect, getType) {
  effect.toString = getType;
}

function dispatcher(value, dispatchDefault, dispatchHook, isBatch = false) {
  if (!isAction(value)) {
    if (undefined !== "production") {
      console.warn(`Not an action`, value);
    }

    return value;
  }

  var dispatch = typeof dispatchHook === 'function' ? dispatchHook : dispatchDefault;
  var result = dispatch(value);
  return result;
}

function isAction(data) {
  return typeof data === 'object' && data != null && typeof data.type === 'string';
}

function watchFailWarn({
  domainName,
  name,
  mode
}, error) {
  switch (mode) {
    case 'throw':
      {
        throw error;
      }

    case 'warn':
      {
        watchWarn(domainName, name, error);
      }

    case 'off':
    default:
      {
        return;
      }
  }
}

var getWarn = ({
  domainName,
  name,
  mode
}) => function never(props) {
  switch (mode()) {
    case 'throw':
      {
        return Promise.reject(implError(domainName, name, props));
      }

    case 'warn':
      {
        implWarn(domainName, name, props);
      }

    case 'off':
    default:
      {
        return new Promise(() => {});
      }
  }
};

function watchWarn(domainName, name, error) {
  console.warn(`


  Watch function should not fail

  Effect:
    ${name}
  Domain:
    ${domainName}
  Error:
    ${error}

`);
}

function implWarn(domainName, name, props) {
  console.warn(`


  Running an effect without implementation

  Name:
    ${name}
  Domain:
    ${domainName}
  Arguments:
    ${props}


`);
}

function implError(domainName, name, props) {
  var message = `Running an effect without implementation

  Name:
    ${name}
  Domain:
    ${domainName}
  Arguments:
    ${props}


`;
  return new Error(message);
}
//# sourceMappingURL=effect.js.map