"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.effectorMiddleware = effectorMiddleware;

var _config = require("./config");

function effectorMiddleware({
  dispatch
}) {
  var plainMap = new Map();
  var config = {
    plain: plainMap
  };
  var isActive = true;
  return next => action => {
    if (!isActive) {
      return action;
    }

    if (!(typeof action === 'object' && action != null)) {
      return next(action);
    }

    if (typeof action.type !== 'string') {
      return action;
    }

    if (action.type === _config.PING) {
      return {
        type: _config.PONG,
        payload: config
      };
    }

    if (action.type === _config.HALT) {
      isActive = false;
      plainMap.clear();
      return next(action);
    }

    var handler = plainMap.get(action.type);

    if (typeof action.send === 'function') {
      action.send(dispatch);
      return action;
    }

    if (!(typeof action.meta === 'object' && action.meta != null)) action.meta = {};
    if (action.meta.passed === true) return action;
    action.meta.passed = true;
    var isFromHandler = action.meta.plain === true;
    var result = next(action);

    if (handler && !isFromHandler) {
      handler(action.payload);
    }

    return result;
  };
}
//# sourceMappingURL=middleware.js.map